'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var _rollupPluginBabelHelpers = require('../_virtual/_rollupPluginBabelHelpers.js');
var React = require('react');
var useFieldApi = require('./useFieldApi.js');
var useRelevance = require('./useRelevance.js');
var useFieldState = require('./useFieldState.js');
var useFormController = require('./useFormController.js');
var useCursorPosition = require('./useCursorPosition.js');
var Context = require('../Context.js');
var utils = require('../utils.js');
var debug = require('../debug.js');
var useUpdateEffect = require('./useUpdateEffect.js');
var useScope = require('./useScope.js');
var useFieldSubscription = require('./useFieldSubscription.js');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var React__default = /*#__PURE__*/_interopDefaultLegacy(React);

var _excluded = ["id", "type", "name", "onBlur", "onChange", "onFocus", "onNativeChange", "onValueChange", "validate", "asyncValidate", "validateModified", "gatherData", "yupSchema", "multiple", "field", "keep", "keepState", "keepStateIfRelevant", "remember", "inputRef", "inputRefs", "relevant", "defaultValue", "initialValue", "autocomplete", "showErrorIfError", "showErrorIfTouched", "showErrorIfDirty", "formatter", "parser", "clean", "mask", "maintainCursor", "required", "noFalsy", "minimum", "maximum", "minLength", "maxLength", "pattern", "allowEmptyString", "emptyValue", "disabled", "gatherOnMount", "gatherOnBlur", "validateOnMount", "modifyOnMount", "validateOn", "maskOnBlur", "validateWhen", "formatterDependencies", "formController", "initialize", "errorMessage", "initializeValueIfPristine", "relevanceWhen", "relevanceDeps", "validateDeps", "evaluate", "evaluateWhen"];
var logger = debug.Debug('informed:useField' + '\t');

/* ----------------------- useField ----------------------- */
var useField = function useField(_ref) {
  var _ref2;
  var id = _ref.id,
    type = _ref.type,
    userName = _ref.name,
    onBlur = _ref.onBlur,
    onChange = _ref.onChange,
    onFocus = _ref.onFocus,
    onNativeChange = _ref.onNativeChange,
    onValueChange = _ref.onValueChange,
    validationFunc = _ref.validate,
    asyncValidate = _ref.asyncValidate,
    userValidateModified = _ref.validateModified,
    gatherData = _ref.gatherData,
    yupSchema = _ref.yupSchema,
    multiple = _ref.multiple,
    field = _ref.field,
    keep = _ref.keep,
    userKeepState = _ref.keepState,
    userKeepStateIfRelevant = _ref.keepStateIfRelevant,
    remember = _ref.remember,
    inputRef = _ref.inputRef,
    inputRefs = _ref.inputRefs,
    relevant = _ref.relevant,
    defaultValue = _ref.defaultValue,
    userInitialValue = _ref.initialValue,
    userAutocomplete = _ref.autocomplete,
    userShowErrorIfError = _ref.showErrorIfError,
    userShowErrorIfTouched = _ref.showErrorIfTouched,
    userShowErrorIfDirty = _ref.showErrorIfDirty,
    formatter = _ref.formatter,
    parser = _ref.parser,
    clean = _ref.clean,
    mask = _ref.mask,
    userMaintainCursor = _ref.maintainCursor,
    required = _ref.required,
    noFalsy = _ref.noFalsy,
    minimum = _ref.minimum,
    maximum = _ref.maximum,
    minLength = _ref.minLength,
    maxLength = _ref.maxLength,
    pattern = _ref.pattern,
    userAllowEmptyString = _ref.allowEmptyString,
    emptyValue = _ref.emptyValue,
    userDisabled = _ref.disabled,
    gatherOnMount = _ref.gatherOnMount,
    gatherOnBlur = _ref.gatherOnBlur,
    userValidateOnMount = _ref.validateOnMount,
    modifyOnMount = _ref.modifyOnMount,
    userValidateOn = _ref.validateOn,
    maskOnBlur = _ref.maskOnBlur,
    _ref$validateWhen = _ref.validateWhen,
    validateWhen = _ref$validateWhen === void 0 ? [] : _ref$validateWhen,
    _ref$formatterDepende = _ref.formatterDependencies,
    formatterDependencies = _ref$formatterDepende === void 0 ? [] : _ref$formatterDepende,
    userFormController = _ref.formController,
    initialize = _ref.initialize,
    errorMessage = _ref.errorMessage,
    initializeValueIfPristine = _ref.initializeValueIfPristine,
    _ref$relevanceWhen = _ref.relevanceWhen,
    relevanceWhen = _ref$relevanceWhen === void 0 ? [] : _ref$relevanceWhen,
    _ref$relevanceDeps = _ref.relevanceDeps,
    relevanceDeps = _ref$relevanceDeps === void 0 ? [] : _ref$relevanceDeps,
    _ref$validateDeps = _ref.validateDeps,
    validateDeps = _ref$validateDeps === void 0 ? [] : _ref$validateDeps;
    _ref.evaluate;
    _ref.evaluateWhen;
    var userProps = _rollupPluginBabelHelpers.objectWithoutProperties(_ref, _excluded);
  // For backwards compatability
  var n = userName !== null && userName !== void 0 ? userName : field;

  // Because it could be scoped
  var name = useScope.useScope(n);

  // Get scoped context
  var scope = React.useContext(Context.ScopeContext);
  if (!name) {
    console.warn('name is a required prop!!!!');
  }

  // Default to maintain cursor whenever formatter is passed
  var maintainCursor = userMaintainCursor !== null && userMaintainCursor !== void 0 ? userMaintainCursor : !!formatter;

  // Grab the form controller
  var formController = userFormController !== null && userFormController !== void 0 ? userFormController : useFormController.useFormController();

  // Get any options
  var autocomplete = userAutocomplete !== null && userAutocomplete !== void 0 ? userAutocomplete : formController.options.current.autocomplete;
  var showErrorIfError = userShowErrorIfError !== null && userShowErrorIfError !== void 0 ? userShowErrorIfError : formController.options.current.showErrorIfError;
  var showErrorIfTouched = userShowErrorIfTouched !== null && userShowErrorIfTouched !== void 0 ? userShowErrorIfTouched : formController.options.current.showErrorIfTouched;
  var showErrorIfDirty = userShowErrorIfDirty !== null && userShowErrorIfDirty !== void 0 ? userShowErrorIfDirty : formController.options.current.showErrorIfDirty;
  var validateOnMount = userValidateOnMount !== null && userValidateOnMount !== void 0 ? userValidateOnMount : formController.options.current.validateOnMount;
  var validateOn = userValidateOn !== null && userValidateOn !== void 0 ? userValidateOn : formController.options.current.validateOn;
  var keepState = userKeepState !== null && userKeepState !== void 0 ? userKeepState : formController.options.current.keepState;
  var keepStateIfRelevant = userKeepStateIfRelevant !== null && userKeepStateIfRelevant !== void 0 ? userKeepStateIfRelevant : formController.options.current.keepStateIfRelevant;
  var allowEmptyString = userAllowEmptyString !== null && userAllowEmptyString !== void 0 ? userAllowEmptyString : formController.options.current.allowEmptyStrings;
  var validateModified = userValidateModified !== null && userValidateModified !== void 0 ? userValidateModified : formController.options.current.validateModified;
  var disabled = (_ref2 = userDisabled !== null && userDisabled !== void 0 ? userDisabled : formController.disabled) !== null && _ref2 !== void 0 ? _ref2 : formController.options.current.disabled;

  // For getting initialValue
  var getInitialValue = function getInitialValue() {
    var _ref3;
    return (_ref3 = userInitialValue !== null && userInitialValue !== void 0 ? userInitialValue : formController.getInitialValue(name)) !== null && _ref3 !== void 0 ? _ref3 : defaultValue;
  };

  // Grab the initial value
  var _useState = React.useState(function () {
      return getInitialValue();
    }),
    _useState2 = _rollupPluginBabelHelpers.slicedToArray(_useState, 1),
    initialValue = _useState2[0];

  // Hook onto the field api
  // Note: we already scoped above so we pass false here
  var fieldApi = useFieldApi.useFieldApi(name, false);

  // For multistep
  var inMultistep = React.useContext(Context.MultistepStepContext);

  // For relevance
  var isRelevant = useRelevance.useRelevance({
    name: name,
    relevant: relevant,
    relevanceWhen: relevanceWhen,
    relevanceDeps: relevanceDeps
  });

  // If we live in `Relevant`
  var relevantContext = React.useContext(Context.RelevanceContext);

  // Create ref
  var internalRef = React.useRef(null);
  var ref = React__default["default"].useMemo(function () {
    return inputRef || internalRef;
  }, []);

  // Create Id for field
  var _useState3 = React.useState(function () {
      return id || utils.uuidv4();
    }),
    _useState4 = _rollupPluginBabelHelpers.slicedToArray(_useState3, 1),
    fieldId = _useState4[0];

  // Create reference function for if it changes
  var validationFuncRef = React.useRef();
  validationFuncRef.current = validationFunc;

  // Generate validation function
  var validate = React.useMemo(function () {
    return utils.generateValidationFunction(validationFuncRef, yupSchema, {
      required: required,
      noFalsy: noFalsy,
      minimum: minimum,
      maximum: maximum,
      minLength: minLength,
      maxLength: maxLength,
      pattern: pattern,
      getErrorMessage: function getErrorMessage(key) {
        return formController.getErrorMessage(key, name);
      },
      validateModified: validateModified,
      fieldApi: fieldApi,
      formController: formController,
      scope: scope
    });
  }, [required, minimum, maximum, minLength, maxLength, pattern, noFalsy]);

  // Create meta object
  var meta = {
    name: name,
    type: type,
    onBlur: onBlur,
    onChange: onChange,
    onFocus: onFocus,
    onNativeChange: onNativeChange,
    initialValue: initialValue,
    keep: keep,
    remember: remember,
    keepState: keepState,
    keepStateIfRelevant: keepStateIfRelevant,
    initializeValueIfPristine: initializeValueIfPristine,
    fieldApi: fieldApi,
    getInitialValue: getInitialValue,
    formatter: formatter,
    parser: parser,
    clean: clean,
    mask: mask,
    validate: validate,
    yupSchema: yupSchema,
    validateOn: validateOn !== null && validateOn !== void 0 ? validateOn : 'blur',
    validateOnMount: validateOnMount,
    validateWhen: validateWhen,
    showErrorIfError: showErrorIfError,
    showErrorIfTouched: showErrorIfTouched,
    showErrorIfDirty: showErrorIfDirty,
    maskOnBlur: maskOnBlur,
    asyncValidate: asyncValidate,
    gatherData: gatherData,
    initialize: initialize,
    errorMessage: errorMessage,
    allowEmptyString: allowEmptyString,
    emptyValue: emptyValue,
    gatherOnMount: gatherOnMount,
    gatherOnBlur: gatherOnBlur,
    fieldRef: ref,
    modifyOnMount: modifyOnMount
  };
  var metaRef = React.useRef(meta);
  metaRef.current = meta;

  // Before we hook into field state initialize the field
  React.useState(function () {
    // Only initialize if relevant
    if (isRelevant) {
      var metaInfo = metaRef.current;
      logger('Initialize', metaInfo.name);
      formController.initialize(metaInfo.name, metaRef, false);
    }
  });

  // Hook onto the field state
  // Note: we already scoped above so we pass false here
  // Note: its important this call is here ( below where we call initialize ) so the state can be set up before we start using it
  var fieldState = useFieldState.useFieldState(name, false);

  // Setup cursor position tracking
  var _useCursorPosition = useCursorPosition.useCursorPosition({
      value: fieldState.value,
      inputRef: ref,
      maintainCursor: maintainCursor,
      inputRefs: inputRefs
    }),
    setCursor = _useCursorPosition.setCursor,
    setCursorOffset = _useCursorPosition.setCursorOffset;

  // Add to meta
  metaRef.current.setCursorOffset = setCursorOffset;
  metaRef.current.setCursor = setCursor;

  // Register
  React.useEffect(function () {
    if (isRelevant) {
      // We already initialized before the render so the input exists in the form state, we need to redo after the render
      logger('Register', name, metaRef.current);
      formController.register(name, metaRef);
      logger('Second Initialize', name);
      formController.initialize(name, metaRef, false);
    }
    return function () {
      logger('De-Register', name, metaRef.current);
      formController.deregister(name);
    };
  }, [name]);

  // Cleanup on irrelivant
  // Note: important to use the update effect so we dont call this on first render
  useUpdateEffect.useUpdateEffect(function () {
    // The info may have changed, grab it from the ref
    var metaInfo = metaRef.current;
    if (!isRelevant && !keepState) {
      logger('RELEVANT REMOVING', metaInfo.name);
      formController.remove(metaInfo.name, metaInfo.keep, metaInfo);
      logger('RELEVANT De-Register', metaInfo.name);
      formController.deregister(metaInfo.name);
    }
    if (isRelevant) {
      logger('RELEVANT register', metaInfo.name);
      formController.register(metaInfo.name, metaRef);
      logger('RELEVANT Initialize', metaInfo.name);
      formController.initialize(metaInfo.name, metaRef);
    }
  }, [isRelevant]);

  // Cleanup on un-mount
  React.useEffect(function () {
    return function () {
      var keepIt = false;

      // The info may have changed, grab it from the ref
      var metaInfo = metaRef.current;
      logger('CLEANUP REMOVING', metaInfo.name);

      // Always keep it if this is passed
      if (metaInfo.keepState) {
        keepIt = true;
      }
      // Need to check relevance because we DONT
      // want to keep if we are irrelivant
      else if (relevantContext && !relevantContext.relevant()) {
        keepIt = false;
      }
      // If we make it here we must be relevant so check keepStateIfRelevant
      else if (keepStateIfRelevant) {
        keepIt = true;
      }
      // If its a multistep then we also want to keep it
      else if (inMultistep) {
        keepIt = true;
      }
      if (!keepIt) {
        formController.remove(metaInfo.name, metaInfo.keep, metaInfo);
      }
    };
  }, []);
  useUpdateEffect.useUpdateEffect(function () {
    formController.reformat(metaRef.current.name);
  }, _rollupPluginBabelHelpers.toConsumableArray(formatterDependencies));

  // Note im not adding this yet as I need to figure out how to solve issue with array fields when you remove 1 [0, 1, 2] and 2 becomes 1
  // useUpdateEffect(
  //   () => {
  //     // If the form is pristine then reset it when we get new initial values !
  //     const pristine = fieldApi.getPristine();
  //     if (pristine) {
  //       fieldApi.reset();
  //     }
  //   },
  //   [userInitialValue, defaultValue]
  // );

  useFieldSubscription.useFieldSubscription('field-value', validateWhen, function (target) {
    logger("revalidating for ".concat(metaRef.current.name, " because of ").concat(target));
    formController.validateField(metaRef.current.name);
  });
  useFieldSubscription.useFieldSubscription('field-value', [name], function (target) {
    if (onValueChange) {
      onValueChange(formController.getFieldState(target));
    }
  }, false // No scope as we are already scoped
  );
  useUpdateEffect.useUpdateEffect(function () {
    logger("revalidating for ".concat(metaRef.current.name, " because of deps change"));
    formController.validateField(metaRef.current.name);
  }, validateDeps);
  var render = function render(children) {
    return isRelevant ? children : null;
  };
  var changeHandler = utils.generateOnChange({
    fieldType: type,
    setValue: fieldApi.setValue,
    multiple: multiple,
    ref: ref
  });
  var blurHandler = utils.generateOnBlur({
    setTouched: fieldApi.setTouched
  });
  var focusHandler = utils.generateOnFocus({
    setFocused: fieldApi.setFocused
  });
  var hookedValue = utils.generateValue({
    fieldType: type,
    maskedValue: fieldState.maskedValue,
    multiple: userProps.multiple,
    value: fieldState.value
  });
  var recombinedUserProps = _rollupPluginBabelHelpers.objectSpread2({
    id: fieldId,
    name: name,
    // ref,
    type: type,
    multiple: multiple,
    autoComplete: autocomplete,
    disabled: disabled,
    required: required,
    min: minimum,
    max: maximum,
    minLength: minLength,
    maxLength: maxLength,
    pattern: pattern
  }, userProps);

  // const order66 = new Date('2022-06-08T04:20Z');

  // if (new Date() > order66) {
  //   recombinedUserProps.placeholder = 'Hello World';
  // }

  return {
    fieldState: fieldState,
    fieldApi: fieldApi,
    userProps: recombinedUserProps,
    informed: {
      onChange: changeHandler,
      onBlur: blurHandler,
      onFocus: focusHandler,
      value: hookedValue
    },
    ref: ref,
    render: render
  };
};

exports.useField = useField;
